# Copyright (c) 2017 FlashX, LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from __future__ import print_function
from six.moves import input
import ctypes
import os


class ExitCLI(Exception):
    """Custom Exception when exit with 1 should happen"""
    pass


def ask_question(question):
    """Method to ask the user a yes/no question

    Args:
        question(str): A question to ask the user

    Returns:
        bool: True if yes, False if no
    """
    valid_response = {"yes": True, "y": True, "no": False, "n": False}

    while True:
        print("{} [y/n]: ".format(question), end="")
        choice = input().lower().strip()
        if choice in valid_response:
            return valid_response[choice]
        else:
            print("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")


def is_running_as_admin():
    """Method to check if the python script is running as an administrator

    Returns:
        bool
    """
    try:
        is_admin = os.getuid() == 0
    except AttributeError:
        is_admin = ctypes.windll.shell32.IsUserAnAdmin()

    return is_admin



# Copyright (c) 2017 FlashX, LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import sys
import platform
from docker.errors import APIError, ImageNotFound, NotFound
import os
import webbrowser
import time
import requests
import uuid

from gigantumcli.dockerinterface import DockerInterface
from gigantumcli.changelog import ChangeLog
from gigantumcli.utilities import ask_question, ExitCLI, is_running_as_admin


def _cleanup_containers() -> None:
    """Method to clean up gigantum managed containers, stopping if needed.

    Note: this method is the only place removal of containers should occur

    Returns:
        None
    """
    docker = DockerInterface()

    # Stop any project containers
    for container in docker.client.containers.list():
        if "gmlb-" in container.name:
            _, user, owner, project_name = container.name.split('-', 3)
            print('- Cleaning up container for Project: {}'.format(project_name))
            container.stop()
            container.remove()

    # Stop app container
    try:
        app_container = docker.client.containers.get("gigantum.labmanager-edge")

        print('- Cleaning up Gigantum Client container')
        app_container.stop()
        app_container.remove()
    except NotFound:
        pass

    try:
        app_container = docker.client.containers.get("gigantum.labmanager")

        print('- Cleaning up Gigantum Client container')
        app_container.stop()
        app_container.remove()
    except NotFound:
        pass


def install(image_name):
    """Method to install the Gigantum Image

    Args:
        image_name(str): Image name, including repository and namespace (e.g. gigantum/labmanager)


    """
    # Make sure user is not root
    if is_running_as_admin():
        raise ExitCLI("Do not run `gigantum install` as root.")

    docker = DockerInterface()

    try:
        try:
            # Check to see if the image has already been pulled
            docker.client.images.get(image_name)
            raise ExitCLI("** Gigantum Client image already installed. Run `gigantum update` instead.")

        except ImageNotFound:
            # Pull for the first time
            print("\nDownloading and installing the Gigantum Client Docker Image. Please wait...\n")
            image = docker.client.images.pull(image_name, 'latest')

    except APIError:
        msg = "ERROR: failed to pull image!"
        msg += "\n- Are you signed into DockerHub?"
        msg += "\n- Do you have access to {}? If not, contact Gigantum.".format(image_name)
        msg += "\n    - Can test by going here: https://hub.docker.com/r/gigantum/labmanager/"
        msg += "\n    - If you see `404 Not Found`, request access from Gigantum\n"
        raise ExitCLI(msg)

    short_id = image.short_id.split(':')[1]
    print("\nSuccessfully pulled {}:{}\n".format(image_name, short_id))


def update(image_name, tag=None):
    """Method to update the existing image, warning about changes before accepting

    Args:
        image_name(str): Image name, including repository and namespace (e.g. gigantum/labmanager)
        tag(str): Tag to pull if you wish to override `latest`

    Returns:
        None
    """
    # Make sure user is not root
    if is_running_as_admin():
        raise ExitCLI("Do not run `gigantum update` as root.")

    docker = DockerInterface()

    try:
        cl = ChangeLog()
        if "edge" not in image_name:
            # Normal install, so do checks
            if not tag:
                # Trying to update to the latest version
                tag = 'latest'

                # Get id of current labmanager install
                try:
                    current_image = docker.client.images.get("{}:latest".format(image_name))
                except ImageNotFound:
                    raise ExitCLI("Gigantum Client image not yet installed. Run 'gigantum install' first.")
                short_id = current_image.short_id.split(':')[1]

                # Check if there is an update available
                if not cl.is_update_available(short_id):
                    print("Latest version already installed.")
                    sys.exit(0)

            # Get Changelog info for the latest or specified version
            try:
                print(cl.get_changelog(tag))
            except ValueError as err:
                raise ExitCLI(err)
        else:
            # Edge build, set tag if needed
            if not tag:
                # Trying to update to the latest version
                tag = 'latest'

        # Make sure user wants to pull
        if ask_question("Are you sure you want to update?"):
            # Pull
            print("\nDownloading and installing the Gigantum Client Docker Image. Please wait...\n")
            image = docker.client.images.pull(image_name, tag)

            # If pulling not truly latest, force to latest
            if tag != 'latest':
                print("Tagging explicit version {} with latest".format(tag))
                docker.client.api.tag('{}:{}'.format(tag, image_name), image_name, 'latest')
        else:
            raise ExitCLI("Update cancelled")
    except APIError:
        msg = "ERROR: failed to pull image!"
        msg += "\n- Are you signed into DockerHub?"
        msg += "\n- Do you have access to {}? If not, contact Gigantum.".format(image_name)
        msg += "\n    - Can test by going here: https://hub.docker.com/r/gigantum/labmanager/"
        msg += "\n    - If you see `404 Not Found`, request access\n"
        raise ExitCLI(msg)

    short_id = image.short_id.split(':')[1]
    print("\nSuccessfully pulled {}:{}\n".format(image_name, short_id))


def _check_for_api(launch_browser=False, timeout=5):
    """Helper method to check for the API to be live for up to 15 seconds and then optionally launch a browser window

    Args:
        launch_browser(bool): flag indicating if the browser should be launched on success == True
        timeout(int): Number of seconds to wait for the API before returning
    Returns:
        bool: flag indicating if the API is ready
    """
    time.sleep(1)
    success = False
    for _ in range(timeout):
        try:
            resp = requests.get("http://localhost:10000/api/ping?v={}".format(uuid.uuid4().hex))

            if resp.status_code == 200:
                success = True
                break
        except requests.exceptions.ConnectionError:
            # allow connection errors, which mean the API isn't up yet.
            pass

        # Sleep for 1 sec and increment counter
        time.sleep(1)

    if success is True and launch_browser is True:
        time.sleep(1)
        # If here, things look OK. Start browser
        webbrowser.open_new("http://localhost:10000")

    return success


def start(image_name, tag=None):
    """Method to start the application

    Args:
        tag(str): Tag to run, defaults to latest
        image_name(str): Image name, including repository and namespace (e.g. gigantum/labmanager)

    Returns:
        None 
    """
    # Make sure user is not root
    if is_running_as_admin():
        raise ExitCLI("Do not run `gigantum start` as root.")

    print("Verifying Docker is available...")
    # Check if Docker is running
    docker = DockerInterface()

    if not tag:
        # Trying to update to the latest version
        tag = 'latest'

    # Check if working dir exists
    working_dir = os.path.join(os.path.expanduser("~"), "gigantum")
    if not os.path.exists(working_dir):
        os.makedirs(working_dir)

    # Check if application has been installed
    try:
        docker.client.images.get("{}:{}".format(image_name, tag))
    except ImageNotFound:
        raise ExitCLI("Gigantum Client image not found. Did you run `gigantum install` yet?")

    # Check to see if already running
    try:
        if _check_for_api(launch_browser=False, timeout=1):
            print("Client already running on http://localhost:10000")
            _check_for_api(launch_browser=True)
            raise ExitCLI("If page does not load, restart by running `gigantum stop` and then `gigantum start` again")

        # remove any lingering gigantum managed containers
        _cleanup_containers()

    except NotFound:
        # If here, the API isn't running and an older container isn't lingering, so just move along.
        pass

    # Start
    port_mapping = {'10000/tcp': 10000}

    # Make sure the container-container share volume exists
    if not docker.share_volume_exists():
        docker.create_share_volume()

    volume_mapping = {docker.share_vol_name: {'bind': '/mnt/share', 'mode': 'rw'}}

    if platform.system() == 'Windows':
        # windows docker has the following eccentricities
        # no user ids, WINDOWS_HOST env var, /C/a/b/ format for volume C:\\a\\b
        environment_mapping = {'HOST_WORK_DIR': docker.dockerize_volume_path(working_dir),
                               'WINDOWS_HOST': 1}
        volume_mapping[docker.dockerize_volume_path(working_dir)] = {'bind': '/mnt/gigantum', 'mode': 'rw'}

    elif platform.system() == 'Darwin':
        # For macOS, use the cached mode for improved performance
        environment_mapping = {'HOST_WORK_DIR': working_dir,
                               'LOCAL_USER_ID':  os.getuid()}
        volume_mapping[working_dir] = {'bind': '/mnt/gigantum', 'mode': 'cached'}
    else:
        # For anything else, just use default mode.
        environment_mapping = {'HOST_WORK_DIR': working_dir,
                               'LOCAL_USER_ID':  os.getuid()}
        volume_mapping[working_dir] = {'bind': '/mnt/gigantum', 'mode': 'rw'}

    volume_mapping['/var/run/docker.sock'] = {'bind': '/var/run/docker.sock', 'mode': 'rw'}

    container = docker.client.containers.run(image="{}:{}".format(image_name, tag),
                                             detach=True,
                                             name=image_name.replace("/", "."),
                                             init=True,
                                             ports=port_mapping,
                                             volumes=volume_mapping,
                                             environment=environment_mapping)
    print("Starting, please wait...")
    time.sleep(1)

    # Make sure volumes have mounted properly, by checking for the log file for up to 30 seconds
    success = False
    for _ in range(30):
        if os.path.exists(os.path.join(working_dir, '.labmanager', 'logs', 'labmanager.log')):
            success = True
            break

        # Sleep for 1 sec and increment counter
        time.sleep(1)

    if not success:
        msg = "\n\nWorking directory failed to mount! Have you granted Docker access to your user directory?"
        msg = msg + " \nIn both Docker for Mac and Docker for Windows this should be shared by default, but may require"
        msg = msg + " a confirmation from the user."
        msg = msg + "\n\nRun `gigantum stop`, verify your OS and Docker versions are supported, the allowed Docker"
        msg = msg + " volume share locations include `{}`, and try again.".format(working_dir)
        msg = msg + "\n\nIf this problem persists, contact support."

        # Stop and remove the container
        container.stop()
        container.remove()

        raise ExitCLI(msg)

    # Wait for API to be live before opening the user's browser
    if not _check_for_api(launch_browser=True, timeout=15):
        msg = "\n\nGigantum client failed to start! Try restarting Docker and then start again."

        # Stop and remove the container
        container.stop()
        container.remove()

        raise ExitCLI(msg)


def stop():
    """Method to stop all containers

    Returns:
        None
    """
    if ask_question("Stop all Gigantum managed containers? MAKE SURE YOU HAVE SAVED YOUR WORK FIRST!"):
        # remove any lingering gigantum managed containers
        _cleanup_containers()
    else:
        raise ExitCLI("Stop command cancelled")


def feedback():
    """Method to throw up a browser to provide feedback

    Returns:
        None
    """
    feedback_url = "https://feedback.gigantum.com"
    print("You can provide feedback here: {}".format(feedback_url))
    webbrowser.open_new(feedback_url)



# Copyright (c) 2017 FlashX, LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import argparse
from gigantumcli.actions import install, update, start, stop, feedback, ExitCLI
import sys


def main():
    # Setup supported components and commands
    actions = {"install": "Install the Gigantum Client Docker Image",
               "update": "Update the Gigantum Client Docker Image",
               "start": "Start the Client",
               "stop": "Stop the Client",
               "feedback": "Open a web page to provide feedback"
               }

    help_str = ""
    for action in actions:
        help_str = "{}  - {}: {}\n".format(help_str, action, actions[action])

    description_str = "Command Line Interface for the local Gigantum Client application\n\n"
    description_str = description_str + "The following actions are supported:\n\n{}".format(help_str)

    parser = argparse.ArgumentParser(description=description_str,
                                     formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument("--tag", "-t",
                        default=None,
                        metavar="<tag>",
                        help="Image Tag to override the 'latest' Docker Image when updating")

    parser.add_argument("--edge", "-e",
                        action='store_true',
                        help="Optional flag indicating if the edge version should be used."
                             " Applicable to install, update, and start commands. You must have access to this image.")
    parser.add_argument("action", help="Action to perform")

    args = parser.parse_args()

    if not args.edge:
        image_name = 'gigantum/labmanager'
    else:
        image_name = 'gigantum/labmanager-edge'

    try:
        if args.action == "install":
            install(image_name)
        elif args.action == "update":
            update(image_name, args.tag)
        elif args.action == "start":
            start(image_name, args.tag)
        elif args.action == "stop":
            stop()
        elif args.action == "feedback":
            feedback()
        else:
            raise ValueError("Unsupported action `{}` provided. Available actions: {}".format(args.action,
                                                                                              ", ".join(actions.keys())))
    except ExitCLI as err:
        print(err)
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright (c) 2017 FlashX, LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import socket
import json
import os
import docker
import re
import subprocess
import platform
import requests
from docker.errors import NotFound

from gigantumcli.utilities import ask_question, ExitCLI


class DockerInterface(object):
    """Class to provide an interface to Docker"""

    def __init__(self):
        """Constructor"""
        # Get a docker client, or print help on how to install/run docker first
        if self.docker_is_installed():
            # Get client
            self.client = self._get_docker_client()

            if not self.docker_is_running():
                # Docker isn't running
                self._print_running_help()
                raise ExitCLI()
        else:
            # Docker isn't installed
            self._print_installing_help()
            raise ExitCLI()

        # Name of Docker volume used to share between containers
        self.share_vol_name = "labmanager_share_vol"

    @staticmethod
    def _print_running_help():
        """Print help for running docker, taking OS into account

        Returns:
            str
        """
        queried_system = platform.system()
        if queried_system == 'Linux':
            print_cmd = "Docker isn't running. Typically docker runs at startup. Check that `dockerd` is running."
        elif queried_system == 'Darwin':
            print_cmd = "Docker isn't running. Start the Docker for Mac app and try again!"
        elif queried_system == 'Windows':
            print_cmd = "Docker isn't running. Start the Docker CE for Windows app and try again!"
        else:
            raise ValueError("Unsupported OS: {}".format(queried_system))

        print(print_cmd)

    @staticmethod
    def _print_installing_help():
        """Print help for installing docker, taking OS into account

        Returns:
            str
        """
        queried_system = platform.system()
        if queried_system == 'Linux':
            if ask_question("Docker isn't installed. Would you like to try to install it now?"):
                installer_path = os.path.expanduser('~/get-docker.sh')
                resp = requests.get('https://get.docker.com/')
                with open(installer_path, 'wb') as file_handle:
                    file_handle.write(resp.content)

                print_cmd = "An installer script has been downloaded to {}:\n".format(installer_path)
                print_cmd = "{}- Run `sudo sh ~/get-docker.sh`\n".format(print_cmd)
                print_cmd = "{}- Wait for installer to complete\n".format(print_cmd)
                print_cmd = "{}- Run `sudo usermod -aG docker <your-user-name>`\n".format(print_cmd)
                print_cmd = "{}  - This lets you run Docker commands not as root\n".format(print_cmd)
                print_cmd = "{}- Log out and then log back in\n".format(print_cmd)
            else:
                raise ExitCLI("You must install Docker to use the Gigantum application")

        elif queried_system == 'Darwin':
            print_cmd = "Docker isn't installed. Get the Docker for Mac app here: "
            print_cmd = "{}\n\n  https://docs.docker.com/docker-for-mac/install/  \n\n".format(print_cmd)
            print_cmd = "{}- Install the `Stable Channel` version.\n".format(print_cmd)
            print_cmd = "{}- You can change the amount of RAM and CPU allocated to Docker from".format(print_cmd)
            print_cmd = "{} the preferences menu that is available when clicking on the Docker logo".format(print_cmd)
            print_cmd = "{} in the OSX taskbar.\n".format(print_cmd)
            print_cmd = "{}- Be sure to sign-in to DockerHub by clicking on Sign-In\n".format(print_cmd)
            print_cmd = "{}- You don't need to leave Docker running all the time, but it must be".format(print_cmd)
            print_cmd = "{} running before you start the Gigantum application\n".format(print_cmd)

        elif queried_system == 'Windows':
            print_cmd = "Docker isn't installed!\n"
            print_cmd = "{}If you have 64bit Windows 10 Pro, install Docker for Windows app here:".format(print_cmd)
            print_cmd = "{}\n\n  https://docs.docker.com/docker-for-windows/install/  \n\n".format(print_cmd)
            print_cmd = "{}- Install the `Stable Channel` version.\n".format(print_cmd)
            print_cmd = "{}- You can change the amount of RAM and CPU allocated to Docker from".format(print_cmd)
            print_cmd = "{} the preferences menu that is available when clicking on the Docker logo".format(print_cmd)
            print_cmd = "{} in the notification area.\n".format(print_cmd)
            print_cmd = "{}- Be sure to sign-in to DockerHub by clicking on Sign-In\n".format(print_cmd)
            print_cmd = "{}- You don't need to leave Docker running all the time, but it must be".format(print_cmd)
            print_cmd = "{} running before you start the Gigantum application\n".format(print_cmd)
            print_cmd = "{}\nIf you have an old version of Windows, you can still use Docker Toolbox:".format(print_cmd)
            print_cmd = "{}\n\n  https://docs.docker.com/toolbox/overview/  \n\n".format(print_cmd)
        else:
            raise ValueError("Unsupported OS: {}".format(queried_system))

        print(print_cmd)

    @staticmethod
    def docker_is_installed():
        """Method to check if docker is installed"""
        queried_system = platform.system()
        if queried_system in {'Linux', 'Darwin'}:
            check_cmd = "which"
        elif queried_system == 'Windows':
            check_cmd = "where"
        else:
            raise ValueError("Unsupported OS: {}".format(queried_system))

        try:
            subprocess.check_output([check_cmd, 'docker'])
            return True
        except subprocess.CalledProcessError:
            return False

    def docker_is_running(self):
        """Method to check if docker is running"""
        try:
            if not self.client:
                self._get_docker_client()
            return self.client.ping()
        except requests.exceptions.ConnectionError as _:
            return False
        except docker.errors.APIError as _:
            return False

    @staticmethod
    def _get_docker_server_api_version():
        """Retrieve the Docker server API version. """

        socket_path = '/var/run/docker.sock'
        if not os.path.exists(socket_path):
            raise ValueError('No docker.sock on machine (is a Docker server installed?)')

        try:
            socket_connection = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            socket_connection.connect(socket_path)
            socket_connection.send(b'GET http://*/version HTTP/1.1\r\nHost: *\r\n\r\n')
        except ConnectionRefusedError:
            # Assume docker was running at some point to setup env vars, but not running now
            raise ValueError("Could not read from Docker socket")

        response_data = socket_connection.recv(4000)
        content_lines = response_data.decode().split('\r\n')

        version_dict = json.loads(content_lines[-1])
        if 'ApiVersion' not in version_dict.keys():
            raise ValueError('ApiVersion not in Docker version config data')
        else:
            return version_dict['ApiVersion']

    @staticmethod
    def dockerize_volume_path(volpath):
        """Returns a path that can be mounted as a docker volume on windows
            Docker uses non-standard formats for windows mounts.
            This routine converts C:\\a\\b -> /C/a/b on windows and does
            nothing on posix systems.

        Args:
            volpath(str): a python path

        Returns:
            str: path that can be handed to Docker for a volume mount
        """
        # Docker does not take ntpath formatted strings as volume mounts.
        # detect if it's a volume path and rewrite the string.
        if os.path.__name__ == 'ntpath':
            # for windows switch the slashes and then sub the drive letter
            return re.sub('(^[A-Z]):(.*$)', '/\g<1>\g<2>', volpath.replace('\\', '/'))
        else:
            return volpath

    def _get_docker_client(self, check_server_version=True, fallback=True):
        """Return a docker client with proper version to match server API.

        Args:
            check_server_version(bool):
            fallback(bool):

        Returns:
            docker.DockerClient
        """

        if check_server_version:
            try:
                docker_server_api_version = self._get_docker_server_api_version()
                return docker.from_env(version=docker_server_api_version)
            except ValueError as e:
                if fallback:
                    return docker.from_env()
                else:
                    raise e
        else:
            return docker.from_env()

    def share_volume_exists(self):
        """Check if the container-container share volume exists

        Returns:
            bool
        """
        try:
            self.client.volumes.get(self.share_vol_name)
            return True
        except NotFound:
            return False

    def create_share_volume(self):
        """Create the share volume

        Returns:
            None
        """
        self.client.volumes.create(self.share_vol_name)

    def remove_share_volume(self):
        """Remove the share volume

        Returns:
            None
        """
        volume = self.client.volumes.get(self.share_vol_name)
        volume.remove()



from setuptools import setup
from codecs import open
from os import path
from gigantumcli import __version__

# to update
# python setup.py sdist bdist_wheel
# twine upload --skip-existing dist/*

cwd = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(cwd, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# get the dependencies
with open(path.join(cwd, 'requirements.txt'), encoding='utf-8') as f:
    install_requires = f.read().split('\n')

setup(
    name='gigantum',
    version=__version__,

    description='CLI for the Gigantum Platform',
    long_description=long_description,

    install_requires=install_requires,

    author='Gigantum/FlashX, LLC',
    author_email='hello@gigantum.com',

    entry_points={
        'console_scripts': ['gigantum=gigantumcli.cli:main'],
    },
    packages=['gigantumcli'],
    include_package_data=True,

    url='https://github.com/gigantum/gigantum-cli',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 2.7',
    ],
    keywords=[
        'data-science',
        'science',
        'gigantum',
        'open-science'
    ]
)
